﻿/**********************************************************************************************************************
*  Copyright (C), 2022-2026, SMARTLOGIC TECHNOLOGY LTD.
*  File Name: ecpri_test_case102.s.c
*  Create Date: 23/09/20
*  Description: eCPRI Test Case102 Module Source File
*  Change History: 
*      <author>      <time>      <version>   <desc>
*    1. ShangH      23/09/20        1.0       Build this module
**********************************************************************************************************************/


/* -------------------------------------------------- <INC FILE> --------------------------------------------------- */
#include "ecpri_test_case102.h"


/* --------------------------------------------------- <VAR DEF> --------------------------------------------------- */
#ifdef ECS_RFM0
/* eCPRI test ECS_RFM0 flag */
uint32_t g_ecpri_test_flag0 = 0; /* ECS_RFM0: 0xB7E32000(0) */
/* eCPRI test antenna index */
uint32_t g_ecpri_test_ant_id = 0; /* ECS_RFM0: 0xB7E32004(1) */
/* eCPRI test symbol index */
uint32_t g_ecpri_test_sym_id = 0; /* ECS_RFM0: 0xB7E32008(2) */

/* eCPRI test ECS_RFM0 result */
EcpriTestResult_t g_ecpri_test_result0 = {0}; /* ECS_RFM0: 0xB7E32010(4) ~ 0xB7E32018(6) */
#endif

#ifdef ECS_RFM1
/* eCPRI test ECS_RFM1 flag */
uint32_t g_ecpri_test_flag1 = 0; /* ECS_RFM1: 0xB7E42000(0) */

/* eCPRI test ECS_RFM1 result */
EcpriTestResult_t g_ecpri_test_result1 = {0}; /* ECS_RFM1: 0xB7E42010(4) ~ 0xB7E42018(6) */
#endif


/* -------------------------------------------------- <FUNC DEF> --------------------------------------------------- */
/**********************************************************************************************************************
*  Function: fh_data_init
*  Description: fronthaul data init
*  Input: none
*  Output: none
*  Return: operation result - success or failure.
*  Others: none
**********************************************************************************************************************/
int32_t fh_data_init(void)
{
#ifdef ECS_RFM0
    ecpri_test_ecs0_init();
#endif

#ifdef ECS_RFM1
    ecpri_test_ecs1_init();
#endif

    return SUCCESS;
}

/**********************************************************************************************************************
*  Function: fh_drv_init
*  Description: fronthaul driver init
*  Input: none
*  Output: none
*  Return: operation result - success or failure.
*  Others: none
**********************************************************************************************************************/
int32_t fh_drv_init(void)
{
#ifdef ECS_RFM1
    /* eCPRI init */
    ecpri_init(ECPRI_OPTION_10G);
#endif

    return SUCCESS;
}

/**********************************************************************************************************************
*  Function: fh_csu_test_init
*  Description: fronthaul CSU test init
*  Input: none
*  Output: none
*  Return: operation result - success or failure.
*  Others: none
**********************************************************************************************************************/
int32_t fh_csu_test_init(void)
{
    return SUCCESS;
}

/**********************************************************************************************************************
*  Function: fh_data_check
*  Description: fronthaul data check
*  Input: 
*    <name>                    <type>                      <desc>
*     times                     uint32_t                    check times
*  Output: none
*  Return: none
*  Others: none
**********************************************************************************************************************/
void fh_data_check(uint32_t times)
{
#ifdef ECS_RFM0
    ecpri_test_ecs0_status_check();
#endif

#ifdef ECS_RFM1
    ecpri_test_ecs1_status_check();
#endif
}

/**********************************************************************************************************************
*  Function: fh_test_case
*  Description: fronthaul test case
*  Input: none
*  Output: none
*  Return: none
*  Others: none
**********************************************************************************************************************/
void fh_test_case(void)
{
    return;
}

#ifdef ECS_RFM0
/**********************************************************************************************************************
*  Function: ecpri_test_ecs0_init
*  Description: eCPRI test ECS_RFM0 init
*  Input: none
*  Output: none
*  Return: none
*  Others: none
**********************************************************************************************************************/
void ecpri_test_ecs0_init(void)
{
    memset_ucp((void *)&g_ecpri_test_result0, 0, sizeof(g_ecpri_test_result0));

    /* send flag init(symbol interrupt) */
    debug_write(ECPRI_DBG_IDX0(64), 0); /* ECS_RFM0: 0xB7E30100(64) */
    /* send length init */
    debug_write(ECPRI_DBG_IDX0(65), 0); /* ECS_RFM0: 0xB7E30104(65) */
    /* send wait flag init */
    debug_write(ECPRI_DBG_IDX0(66), 0); /* ECS_RFM0: 0xB7E30108(66) */

    /* eCPRI symbol interrupt interval check flag init */
    debug_write(ECPRI_DBG_IDX0(68), 0); /* ECS_RFM0: 0xB7E30110(68) */
    /* eCPRI symbol interrupt status check flag init */
    debug_write(ECPRI_DBG_IDX0(69), 0); /* ECS_RFM0: 0xB7E30114(69) */
    /* eCPRI symbol interrupt parameter check flag init */
    debug_write(ECPRI_DBG_IDX0(70), 0); /* ECS_RFM0: 0xB7E30118(70) */

    /* ECS_RFM0 test flag init */
    debug_write(ECPRI_TEST_IDX0(0), g_ecpri_test_flag0); /* ECS_RFM0: 0xB7E32000(0) */
    /* antenna index init */
    debug_write(ECPRI_TEST_IDX0(1), g_ecpri_test_ant_id); /* ECS_RFM0: 0xB7E32004(1) */
    /* symbol index init */
    debug_write(ECPRI_TEST_IDX0(2), g_ecpri_test_sym_id); /* ECS_RFM0: 0xB7E32008(2) */
}

/**********************************************************************************************************************
*  Function: ecpri_test_pl_pkt_proc
*  Description: eCPRI test PL packet process
*  Input: 
*    <name>                    <type>                      <desc>
*     pkt_addr                  uint32_t                    packet address
*     pkt_len                   uint16_t                    packet length
*  Output: none
*  Return: none
*  Others: none
**********************************************************************************************************************/
void ecpri_test_pl_pkt_proc(uint32_t pkt_addr, uint16_t pkt_len)
{
    return;
}

/**********************************************************************************************************************
*  Function: ecpri_test_ecs0_status_check
*  Description: eCPRI test ECS_RFM0 status check
*  Input: none
*  Output: none
*  Return: none
*  Others: none
**********************************************************************************************************************/
void ecpri_test_ecs0_status_check(void)
{
    /* return value */
    int32_t ret = 0;
    /* antenna number */
    uint32_t ant_num = 1;
    /* symbol number */
    uint32_t sym_num = 1;
    /* count index */
    uint8_t i = 0;
    uint8_t j = 0;

    /* get ECS_RFM0 test flag */
    g_ecpri_test_flag0 = do_read_volatile((void *)ECPRI_TEST_ADDR0(0)); /* ECS_RFM0: 0xB7E32000(0) */
    if (0 == g_ecpri_test_flag0)
    {
        return;
    }

    /* get test antenna index */
    g_ecpri_test_ant_id = do_read_volatile((void *)ECPRI_TEST_ADDR0(1)); /* ECS_RFM0: 0xB7E32004(1) */
    if (ECPRI_10G_ANT_NUM <= g_ecpri_test_ant_id)
    {
        ant_num = ECPRI_10G_ANT_NUM;
    }

    /* get test symbol index */
    g_ecpri_test_sym_id = do_read_volatile((void *)ECPRI_TEST_ADDR0(2)); /* ECS_RFM0: 0xB7E32008(2) */
    if (ECPRI_SYM_NUM <= g_ecpri_test_sym_id)
    {
        sym_num = ECPRI_SYM_NUM;
    }

    for (i = 0; i < ant_num; i++)
    {
        for (j = 0; j < sym_num; j++)
        {
            g_ecpri_test_result0.test_times++;

            if ((1 == ant_num) && (1 == sym_num))
            {
                ret = ecpri_sym_irq_send(g_ecpri_test_ant_id, 0, g_ecpri_test_sym_id);
            }
            else if ((1 == ant_num) && (ECPRI_SYM_NUM == sym_num))
            {
                ret = ecpri_sym_irq_send(g_ecpri_test_ant_id, 0, j);
            }
            else if ((ECPRI_10G_ANT_NUM == ant_num) && (1 == sym_num))
            {
                ret = ecpri_sym_irq_send(i, 0, g_ecpri_test_sym_id);
            }
            else
            {
                ret = ecpri_sym_irq_send(i, 0, j);
            }

            if (SUCCESS == ret)
            {
                g_ecpri_test_result0.succ_times++;
            }
            else
            {
                g_ecpri_test_result0.fail_times++;
            }
        }
    }

    /* eCPRI test ECS_RFM0 result update */
    ecpri_test_ecs0_result_update();

    /* clear ECS_RFM0 test flag */
    g_ecpri_test_flag0 = 0;
    debug_write(ECPRI_TEST_IDX0(0), g_ecpri_test_flag0); /* ECS_RFM0: 0xB7E32000(0) */
}

/**********************************************************************************************************************
*  Function: ecpri_test_ecs0_result_update
*  Description: eCPRI test ECS_RFM0 result update
*  Input: none
*  Output: none
*  Return: none
*  Others: none
**********************************************************************************************************************/
void ecpri_test_ecs0_result_update(void)
{
    debug_write(ECPRI_TEST_IDX0(4), g_ecpri_test_result0.test_times); /* ECS_RFM0: 0xB7E32010(4) */
    debug_write(ECPRI_TEST_IDX0(5), g_ecpri_test_result0.succ_times); /* ECS_RFM0: 0xB7E32014(5) */
    debug_write(ECPRI_TEST_IDX0(6), g_ecpri_test_result0.fail_times); /* ECS_RFM0: 0xB7E32018(6) */
}
#endif

#ifdef ECS_RFM1
/**********************************************************************************************************************
*  Function: ecpri_test_ecs1_init
*  Description: eCPRI test ECS_RFM1 init
*  Input: none
*  Output: none
*  Return: none
*  Others: none
**********************************************************************************************************************/
void ecpri_test_ecs1_init(void)
{
    /* antenna index */
    uint8_t ant_id = 0;
    /* slot type */
    uint8_t slot_type = 0;
    /* symbol index */
    uint8_t sym_id = 0;
    /* count index */
    uint16_t i = 0;

    /* eCPRI buffer clear */
    memset_ucp((void *)ECPRI_BUF_ADDR, 0x00, ECPRI_BUF_SIZE);

    /* data conversion */
    for (ant_id = 0; ant_id < ECPRI_10G_ANT_NUM; ant_id++)
    {
        for (sym_id = 0; sym_id < ECPRI_SYM_NUM; sym_id++)
        {
            HTON_L(g_fdd_ecpri_ant_cfg_data[ant_id].fdd_symx[sym_id].fdd_ecpri_head.vlan_tag);
            HTON_S(g_fdd_ecpri_ant_cfg_data[ant_id].fdd_symx[sym_id].fdd_ecpri_head.ether_type);
            HTON_S(g_fdd_ecpri_ant_cfg_data[ant_id].fdd_symx[sym_id].fdd_ecpri_head.payload_size);

            for (i = 0; i < FDD_ECPRI_SYMX_COMP_LEN; i++)
            {
                HTON_L(g_fdd_ecpri_ant_cfg_data[ant_id].fdd_symx[sym_id].fdd_symx_data[i]);
            }
        }
    }

    /* eCPRI antenna config */
    for (ant_id = 0; ant_id < ECPRI_10G_ANT_NUM; ant_id++)
    {
        for (slot_type = 0; slot_type < ECPRI_SLOT_TYPE; slot_type++)
        {
            for (sym_id = 0; sym_id < ECPRI_SYM_NUM; sym_id++)
            {
                ecpri_ant_cfg(ant_id, slot_type, sym_id, 
                              (uint32_t)(&g_fdd_ecpri_ant_cfg_data[ant_id].fdd_symx[sym_id]), FDD_ECPRI_SYMX_PKT_LEN);
            }
        }
    }

    memset_ucp((void *)&g_ecpri_test_result1, 0, sizeof(g_ecpri_test_result1));

    /* ECS_RFM1 test flag init */
    debug_write(ECPRI_TEST_IDX1(0), g_ecpri_test_flag1); /* ECS_RFM1: 0xB7E42000(0) */
}

/**********************************************************************************************************************
*  Function: ecpri_test_ecs1_status_check
*  Description: eCPRI test ECS_RFM1 status check
*  Input: none
*  Output: none
*  Return: none
*  Others: none
**********************************************************************************************************************/
void ecpri_test_ecs1_status_check(void)
{
    /* get ECS_RFM1 test flag */
    g_ecpri_test_flag1 = do_read_volatile((void *)ECPRI_TEST_ADDR1(0)); /* ECS_RFM1: 0xB7E42000(0) */
    if (0 == g_ecpri_test_flag1)
    {
        return;
    }

    /* eCPRI test ECS_RFM1 result update */
    ecpri_test_ecs1_result_update();

    /* clear ECS_RFM1 test flag */
    g_ecpri_test_flag1 = 0;
    debug_write(ECPRI_TEST_IDX1(0), g_ecpri_test_flag1); /* ECS_RFM1: 0xB7E42000(0) */
}

/**********************************************************************************************************************
*  Function: ecpri_test_ecs1_result_update
*  Description: eCPRI test ECS_RFM1 result update
*  Input: none
*  Output: none
*  Return: none
*  Others: none
**********************************************************************************************************************/
void ecpri_test_ecs1_result_update(void)
{
    debug_write(ECPRI_TEST_IDX1(4), g_ecpri_test_result1.test_times); /* ECS_RFM1: 0xB7E42010(4) */
    debug_write(ECPRI_TEST_IDX1(5), g_ecpri_test_result1.succ_times); /* ECS_RFM1: 0xB7E42014(5) */
    debug_write(ECPRI_TEST_IDX1(6), g_ecpri_test_result1.fail_times); /* ECS_RFM1: 0xB7E42018(6) */
}
#endif



