﻿/**********************************************************************************************************************
*  Copyright (C), 2022-2026, SMARTLOGIC TECHNOLOGY LTD.
*  File Name: ecpri_test_case108.s.c
*  Create Date: 24/02/02
*  Description: eCPRI Test Case108 Module Source File
*  Change History: 
*      <author>      <time>      <version>   <desc>
*    1. ShangH      24/02/02        1.0       Build this module
**********************************************************************************************************************/


/* -------------------------------------------------- <INC FILE> --------------------------------------------------- */
#include "ecpri_test_case108.h"


/* --------------------------------------------------- <VAR DEF> --------------------------------------------------- */
#ifdef ECS_RFM0
/* eCPRI test ECS_RFM0 flag */
uint32_t g_ecpri_test_flag0 = 0; /* ECS_RFM0: 0xB7E32000(0) */

/* eCPRI test ECS_RFM0 result */
EcpriTestResult_t g_ecpri_test_result0 = {0}; /* ECS_RFM0: 0xB7E32010(4) ~ 0xB7E32018(6) */
#endif

#ifdef ECS_RFM1
/* eCPRI test ECS_RFM1 flag */
uint32_t g_ecpri_test_flag1 = 0; /* ECS_RFM1: 0xB7E42000(0) */

/* eCPRI test ECS_RFM1 result */
EcpriTestResult_t g_ecpri_test_result1 = {0}; /* ECS_RFM1: 0xB7E42010(4) ~ 0xB7E42018(6) */
#endif


/* -------------------------------------------------- <FUNC DEF> --------------------------------------------------- */
/**********************************************************************************************************************
*  Function: fh_data_init
*  Description: fronthaul data init
*  Input: none
*  Output: none
*  Return: operation result - success or failure.
*  Others: none
**********************************************************************************************************************/
int32_t fh_data_init(void)
{
#ifdef ECS_RFM0
    ecpri_test_ecs0_init();
#endif

#ifdef ECS_RFM1
    ecpri_test_ecs1_init();
#endif

    return SUCCESS;
}

/**********************************************************************************************************************
*  Function: fh_drv_init
*  Description: fronthaul driver init
*  Input: none
*  Output: none
*  Return: operation result - success or failure.
*  Others: none
**********************************************************************************************************************/
int32_t fh_drv_init(void)
{
#ifdef ECS_RFM1
    /* eCPRI init */
    ecpri_init(ECPRI_OPTION_10G);
#endif

    return SUCCESS;
}

/**********************************************************************************************************************
*  Function: fh_csu_test_init
*  Description: fronthaul CSU test init
*  Input: none
*  Output: none
*  Return: operation result - success or failure.
*  Others: none
**********************************************************************************************************************/
int32_t fh_csu_test_init(void)
{
    return SUCCESS;
}

/**********************************************************************************************************************
*  Function: fh_data_check
*  Description: fronthaul data check
*  Input: 
*    <name>                    <type>                      <desc>
*     times                     uint32_t                    check times
*  Output: none
*  Return: none
*  Others: none
**********************************************************************************************************************/
void fh_data_check(uint32_t times)
{
#ifdef ECS_RFM0
    ecpri_test_ecs0_status_check();
#endif

#ifdef ECS_RFM1
    ecpri_test_ecs1_status_check();
#endif
}

/**********************************************************************************************************************
*  Function: fh_test_case
*  Description: fronthaul test case
*  Input: none
*  Output: none
*  Return: none
*  Others: none
**********************************************************************************************************************/
void fh_test_case(void)
{
    return;
}

#ifdef ECS_RFM0
/**********************************************************************************************************************
*  Function: ecpri_test_ecs0_init
*  Description: eCPRI test ECS_RFM0 init
*  Input: none
*  Output: none
*  Return: none
*  Others: none
**********************************************************************************************************************/
void ecpri_test_ecs0_init(void)
{
    memset_ucp((void *)&g_ecpri_test_result0, 0, sizeof(g_ecpri_test_result0));

    /* test flag init */
    debug_write(ECPRI_TEST_IDX0(0), g_ecpri_test_flag0); /* ECS_RFM0: 0xB7E32000(0) */
}

/**********************************************************************************************************************
*  Function: ecpri_test_pl_pkt_proc
*  Description: eCPRI test PL packet process
*  Input: 
*    <name>                    <type>                      <desc>
*     pkt_addr                  uint32_t                    packet address
*     pkt_len                   uint16_t                    packet length
*  Output: none
*  Return: none
*  Others: none
**********************************************************************************************************************/
void ecpri_test_pl_pkt_proc(uint32_t pkt_addr, uint16_t pkt_len)
{
    return;
}

/**********************************************************************************************************************
*  Function: ecpri_test_ecs0_status_check
*  Description: eCPRI test ECS_RFM0 status check
*  Input: none
*  Output: none
*  Return: none
*  Others: none
**********************************************************************************************************************/
void ecpri_test_ecs0_status_check(void)
{
    /* get test flag */
    g_ecpri_test_flag0 = do_read_volatile((void *)ECPRI_TEST_ADDR0(0)); /* ECS_RFM0: 0xB7E32000(0) */
    if (0 == g_ecpri_test_flag0)
    {
        return;
    }

    /* eCPRI test ECS_RFM0 result update */
    ecpri_test_ecs0_result_update();

    /* clear ECS_RFM0 test flag */
    g_ecpri_test_flag0 = 0;
    debug_write(ECPRI_TEST_IDX0(0), g_ecpri_test_flag0); /* ECS_RFM0: 0xB7E32000(0) */
}

/**********************************************************************************************************************
*  Function: ecpri_test_ecs0_result_update
*  Description: eCPRI test ECS_RFM0 result update
*  Input: none
*  Output: none
*  Return: none
*  Others: none
**********************************************************************************************************************/
void ecpri_test_ecs0_result_update(void)
{
    debug_write(ECPRI_TEST_IDX0(4), g_ecpri_test_result0.test_times); /* ECS_RFM0: 0xB7E32010(4) */
    debug_write(ECPRI_TEST_IDX0(5), g_ecpri_test_result0.succ_times); /* ECS_RFM0: 0xB7E32014(5) */
    debug_write(ECPRI_TEST_IDX0(6), g_ecpri_test_result0.fail_times); /* ECS_RFM0: 0xB7E32018(6) */
}
#endif

#ifdef ECS_RFM1
/**********************************************************************************************************************
*  Function: ecpri_test_ecs1_init
*  Description: eCPRI test ECS_RFM1 init
*  Input: none
*  Output: none
*  Return: none
*  Others: none
**********************************************************************************************************************/
void ecpri_test_ecs1_init(void)
{
    /* eCPRI buffer clear */
    memset_ucp((void *)ECPRI_BUF_ADDR, 0x00, ECPRI_BUF_SIZE);

    memset_ucp((void *)&g_ecpri_test_result1, 0, sizeof(g_ecpri_test_result1));

    /* ECS_RFM1 test flag init */
    debug_write(ECPRI_TEST_IDX1(0), g_ecpri_test_flag1); /* ECS_RFM1: 0xB7E42000(0) */
}

/**********************************************************************************************************************
*  Function: ecpri_test_ecs1_status_check
*  Description: eCPRI test ECS_RFM1 status check
*  Input: none
*  Output: none
*  Return: none
*  Others: none
**********************************************************************************************************************/
void ecpri_test_ecs1_status_check(void)
{
    /* eCPRI link status */
    bool link_status = false;

    /* get ECS_RFM1 test flag */
    g_ecpri_test_flag1 = do_read_volatile((void *)ECPRI_TEST_ADDR1(0)); /* ECS_RFM1: 0xB7E42000(0) */
    if (0 == g_ecpri_test_flag1)
    {
        return;
    }

    /* get eCPRI link status */
    link_status = ecpri_link_status_get();
    debug_write(ECPRI_TEST_IDX1(8), (uint32_t)link_status); /* ECS_RFM1: 0xB7E42020(8) */

    /* eCPRI test ECS_RFM1 result update */
    ecpri_test_ecs1_result_update();

    /* clear ECS_RFM1 test flag */
    g_ecpri_test_flag1 = 0;
    debug_write(ECPRI_TEST_IDX1(0), g_ecpri_test_flag1); /* ECS_RFM1: 0xB7E42000(0) */
}

/**********************************************************************************************************************
*  Function: ecpri_test_ecs1_result_update
*  Description: eCPRI test ECS_RFM1 result update
*  Input: none
*  Output: none
*  Return: none
*  Others: none
**********************************************************************************************************************/
void ecpri_test_ecs1_result_update(void)
{
    debug_write(ECPRI_TEST_IDX1(4), g_ecpri_test_result1.test_times); /* ECS_RFM1: 0xB7E42010(4) */
    debug_write(ECPRI_TEST_IDX1(5), g_ecpri_test_result1.succ_times); /* ECS_RFM1: 0xB7E42014(5) */
    debug_write(ECPRI_TEST_IDX1(6), g_ecpri_test_result1.fail_times); /* ECS_RFM1: 0xB7E42018(6) */
}
#endif



