﻿/**********************************************************************************************************************
*  Copyright (C), 2022-2026, SMARTLOGIC TECHNOLOGY LTD.
*  File Name: ecpri_test_case110.s.c
*  Create Date: 23/12/18
*  Description: eCPRI Test Case110 Module Source File
*  Change History: 
*      <author>      <time>      <version>   <desc>
*    1. ShangH      23/12/18        1.0       Build this module
**********************************************************************************************************************/


/* -------------------------------------------------- <INC FILE> --------------------------------------------------- */
#include "ecpri_test_case110.h"


/* --------------------------------------------------- <VAR DEF> --------------------------------------------------- */
#ifdef ECS_RFM0
/* eCPRI test flag */
uint32_t g_ecpri_test_flag = 0; /* ECS_RFM0: 0xB7E32000(0) */

/* eCPRI loopback test result */
EcpriLBTestResult_t g_ecpri_lb_test_result = {0}; /* ECS_RFM0: 0xB7E36010(4100) ~ 0xB7E36018(4102) */
#endif


/* -------------------------------------------------- <FUNC DEF> --------------------------------------------------- */
/**********************************************************************************************************************
*  Function: fh_data_init
*  Description: fronthaul data init
*  Input: none
*  Output: none
*  Return: operation result - success or failure.
*  Others: none
**********************************************************************************************************************/
int32_t fh_data_init(void)
{
#ifdef ECS_RFM0
    ecpri_test_ecs0_init();
#endif

#ifdef ECS_RFM1
    ecpri_test_ecs1_init();
#endif

    return SUCCESS;
}

/**********************************************************************************************************************
*  Function: fh_drv_init
*  Description: fronthaul driver init
*  Input: none
*  Output: none
*  Return: operation result - success or failure.
*  Others: none
**********************************************************************************************************************/
int32_t fh_drv_init(void)
{
#ifdef ECS_RFM1
    /* eCPRI init */
    ecpri_init(ECPRI_OPTION_25G);
#endif

    return SUCCESS;
}

/**********************************************************************************************************************
*  Function: fh_csu_test_init
*  Description: fronthaul CSU test init
*  Input: none
*  Output: none
*  Return: operation result - success or failure.
*  Others: none
**********************************************************************************************************************/
int32_t fh_csu_test_init(void)
{
    return SUCCESS;
}

/**********************************************************************************************************************
*  Function: fh_data_check
*  Description: fronthaul data check
*  Input: 
*    <name>                    <type>                      <desc>
*     times                     uint32_t                    check times
*  Output: none
*  Return: none
*  Others: none
**********************************************************************************************************************/
void fh_data_check(uint32_t times)
{
#ifdef ECS_RFM0
    ecpri_test_ecs0_status_check();
#endif
}

/**********************************************************************************************************************
*  Function: fh_test_case
*  Description: fronthaul test case
*  Input: none
*  Output: none
*  Return: none
*  Others: none
**********************************************************************************************************************/
void fh_test_case(void)
{
    return;
}

#ifdef ECS_RFM0
/**********************************************************************************************************************
*  Function: ecpri_test_ecs0_init
*  Description: eCPRI test ECS_RFM0 init
*  Input: none
*  Output: none
*  Return: none
*  Others: none
**********************************************************************************************************************/
void ecpri_test_ecs0_init(void)
{
    memset_ucp((void *)&g_ecpri_lb_test_result, 0, sizeof(g_ecpri_lb_test_result));

    /* send flag init(symbol interrupt) */
    debug_write(ECPRI_DBG_IDX0(64), 0); /* ECS_RFM0: 0xB7E30100(64) */
    /* send length init */
    debug_write(ECPRI_DBG_IDX0(65), 0); /* ECS_RFM0: 0xB7E30104(65) */
    /* send wait flag init */
    debug_write(ECPRI_DBG_IDX0(66), 0); /* ECS_RFM0: 0xB7E30108(66) */

    /* eCPRI symbol interrupt interval check flag init */
    debug_write(ECPRI_DBG_IDX0(68), 0); /* ECS_RFM0: 0xB7E30110(68) */
    /* eCPRI symbol interrupt status check flag init */
    debug_write(ECPRI_DBG_IDX0(69), 0); /* ECS_RFM0: 0xB7E30114(69) */
    /* eCPRI symbol interrupt parameter check flag init */
    debug_write(ECPRI_DBG_IDX0(70), 0); /* ECS_RFM0: 0xB7E30118(70) */

    /* eCPRI test flag init */
    debug_write(ECPRI_TEST_IDX0(0), g_ecpri_test_flag); /* ECS_RFM0: 0xB7E32000(0) */
}

/**********************************************************************************************************************
*  Function: ecpri_test_pl_pkt_proc
*  Description: eCPRI test PL packet process
*  Input: 
*    <name>                    <type>                      <desc>
*     pkt_addr                  uint32_t                    packet address
*     pkt_len                   uint16_t                    packet length
*  Output: none
*  Return: none
*  Others: none
**********************************************************************************************************************/
void ecpri_test_pl_pkt_proc(uint32_t pkt_addr, uint16_t pkt_len)
{
    /* counter index */
    uint8_t i = 0;
    /* source MAC */
    uint8_t src_mac = 0;
    /* destination MAC */
    uint8_t dst_mac = 0;

    g_ecpri_lb_test_result.pl_stat.test_times++;

    /* input para check */
    if ((NULL == (void *)pkt_addr) || (ECPRI_PKT_SIZE < pkt_len))
    {
        g_ecpri_lb_test_result.pl_stat.fail_times++;
        return;
    }

    /* data temporary storage */
    (void)ape_csu_dma_1D_G2L_ch2ch3_transfer(pkt_addr, 
                                             ECPRI_TEST_ADDR0(4), 
                                             pkt_len, 
                                             1, 
                                             1); /* ECS_RFM0: 0xB7E32010(4) ~ 0xB7E3600C(4099) */

    //__ucps2_synch(0);

    /* exchange source/destination MAC address */
    for (i = 0; i < MAC_ADDR_LEN; i++)
    {
        dst_mac = do_read_volatile_byte((void *)(ECPRI_TEST_ADDR0(4) + i));
        src_mac = do_read_volatile_byte((void *)(ECPRI_TEST_ADDR0(4) + i + MAC_ADDR_LEN));

        //__ucps2_synch(0);

        do_write_byte((void *)(ECPRI_TEST_ADDR0(4) + i), src_mac);
        do_write_byte((void *)(ECPRI_TEST_ADDR0(4) + i + MAC_ADDR_LEN), dst_mac);
    }

    /* eCPRI CSU send packet */
    (void)ecpri_csu_send(ECPRI_TEST_ADDR0(4), pkt_len, 0); /* ECS_RFM0: 0xB7E32010(4) ~ 0xB7E3600C(4099) */

    g_ecpri_lb_test_result.pl_stat.succ_times++;
}

/**********************************************************************************************************************
*  Function: ecpri_test_ecs0_status_check
*  Description: eCPRI test ECS_RFM0 status check
*  Input: none
*  Output: none
*  Return: none
*  Others: none
**********************************************************************************************************************/
void ecpri_test_ecs0_status_check(void)
{
    /* get eCPRI test flag */
    g_ecpri_test_flag = do_read_volatile(ECPRI_TEST_ADDR0(0)); /* ECS_RFM0: 0xB7E32000(0) */
    if (0 == g_ecpri_test_flag)
    {
        return;
    }

    /* eCPRI test ECS_RFM0 result update */
    ecpri_test_ecs0_result_update();

    /* clear eCPRI test flag */
    g_ecpri_test_flag = 0;
    debug_write(ECPRI_TEST_IDX0(0), g_ecpri_test_flag); /* ECS_RFM0: 0xB7E32000(0) */
}

/**********************************************************************************************************************
*  Function: ecpri_test_ecs0_result_update
*  Description: eCPRI test ECS_RFM0 result update
*  Input: none
*  Output: none
*  Return: none
*  Others: none
**********************************************************************************************************************/
void ecpri_test_ecs0_result_update(void)
{
    debug_write(ECPRI_TEST_IDX0(4100), g_ecpri_lb_test_result.pl_stat.test_times); /* ECS_RFM0: 0xB7E36010(4100) */
    debug_write(ECPRI_TEST_IDX0(4101), g_ecpri_lb_test_result.pl_stat.succ_times); /* ECS_RFM0: 0xB7E36014(4101) */
    debug_write(ECPRI_TEST_IDX0(4102), g_ecpri_lb_test_result.pl_stat.fail_times); /* ECS_RFM0: 0xB7E36018(4102) */
}
#endif

#ifdef ECS_RFM1
/**********************************************************************************************************************
*  Function: ecpri_test_ecs1_init
*  Description: eCPRI test ECS_RFM1 init
*  Input: none
*  Output: none
*  Return: none
*  Others: none
**********************************************************************************************************************/
void ecpri_test_ecs1_init(void)
{
    /* eCPRI buffer clear */
    memset_ucp((void *)ECPRI_BUF_ADDR, 0x00, ECPRI_BUF_SIZE);
}
#endif



